﻿Imports C1.WPF.FlexGrid
Imports System.ComponentModel

Class MainWindow
    Public Sub New()
        InitializeComponent()

        '  add numeric column
        Dim c As New Column()
        c.Format = "c"
        c.Header = "Double"
        c.DataType = GetType(System.Nullable(Of Double))
        _flex.Columns.Add(c)

        ' add DateTime column
        c = New Column()
        c.Header = "Date"
        c.Format = "dd - MMM - yy"
        c.DataType = GetType(System.Nullable(Of DateTime))
        _flex.Columns.Add(c)

        ' add Boolean column
        c = New Column()
        c.Header = "Bool"
        c.DataType = GetType(Boolean)
        _flex.Columns.Add(c)

        ' add some rows
        For i As Integer = 0 To 9
            _flex.Rows.Add(New Row())
        Next

        ' set some values
        For i As Integer = 0 To _flex.Rows.Count - 1
            _flex(i, 0) = i
            _flex(i, 1) = DateTime.Today.AddDays(i)
            _flex(i, 2) = i Mod 2 = 0
        Next

        _flex.AllowResizing = AllowResizing.None
        _flex.AllowDragging = AllowDragging.None

        ' custom handling for clicks
        AddHandler _flex.Click, AddressOf _flex_Click

    End Sub

    Private Sub _flex_Click(sender As Object, e As MouseButtonEventArgs)

        Dim ht = _flex.HitTest(e)

        ' sort rows when user clicks a row header
        If ht.CellType = CellType.ColumnHeader Then
            Dim col = _flex.Columns(ht.Column)

            Dim sortDirection = If((TypeOf col.Tag Is ListSortDirection) AndAlso DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Ascending, ListSortDirection.Descending, ListSortDirection.Ascending)
            col.Tag = sortDirection
            'implement sort logic on this column
            SortColumn(col, sortDirection)

            For Each column In _flex.Columns
                If (column.Index <> col.Index) Then
                    column.Tag = Nothing
                End If
            Next
        End If

        ' toggle Boolean cells even when the click is outside the box
        If ht.CellType = CellType.Cell Then
            ' check that it is a Boolean cell
            If _flex.Columns(ht.Column).DataType = GetType(Boolean) Then
                ' check that it is a single cell (not when extending the selection)
                If _flex.Selection.IsSingleCell Then
                    ' update Boolean value
                    Dim value = _flex(ht.Row, ht.Column)
                    _flex(ht.Row, ht.Column) = If(TypeOf value Is Boolean AndAlso CBool(value), False, True)

                    ' we're done with this event
                    e.Handled = True
                End If
            End If
        End If

        'design column column header (post-sort)
        _flex.CellFactory = New SortCellFactory()
    End Sub

    ' sort a column given a sort direction
    Private Sub SortColumn(col As Column, sortDirection As ListSortDirection)
        ' build row list
        Dim list = New List(Of Row)()
        For Each row In _flex.Rows
            list.Add(row)
        Next

        ' sort row list
        list.Sort(New RowComparer(col, sortDirection))

        ' re-populate grid with sorted rows
        _flex.Rows.Clear()
        For Each row In list
            _flex.Rows.Add(row)
        Next
    End Sub
End Class

Public Class RowComparer
    Implements IComparer(Of Row)

    Private _col As Column
    Private _sortDirection As ListSortDirection


    Public Sub New(col As Column, sortDirection As ListSortDirection)
        _col = col
        _sortDirection = sortDirection
    End Sub


    Public Function Compare(x As C1.WPF.FlexGrid.Row, y As C1.WPF.FlexGrid.Row) As Integer Implements System.Collections.Generic.IComparer(Of C1.WPF.FlexGrid.Row).Compare
        Dim v1 = x(_col)
        Dim v2 = y(_col)

        ' compare values
        Dim cmp As Integer = 0
        If v1 Is Nothing AndAlso v2 IsNot Nothing Then
            cmp = -1
        ElseIf v1 IsNot Nothing AndAlso v2 Is Nothing Then
            cmp = +1
        ElseIf TypeOf v1 Is IComparable AndAlso TypeOf v2 Is IComparable Then
            cmp = DirectCast(v1, IComparable).CompareTo(v2)
        End If

        ' honor sort direction
        If _sortDirection = ListSortDirection.Descending Then
            cmp = -cmp
        End If

        ' return result
        Return cmp
    End Function
End Class

Public Class SortCellFactory
    Inherits CellFactory

    Public Overrides Sub CreateColumnHeaderContent(grid As C1FlexGrid, bdr As Border, range As CellRange)

        Dim col = grid.Columns(range.Column)

        MyBase.CreateColumnHeaderContent(grid, bdr, range)
        If Not (col.Tag Is Nothing) Then
            If DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Descending Then

                Dim header As New Grid()

                Dim tb As New TextBlock()
                If bdr.Child IsNot Nothing Then
                    tb = TryCast(bdr.Child, TextBlock)
                End If
                bdr.Child = Nothing
                tb.HorizontalAlignment = HorizontalAlignment.Left
                tb.VerticalAlignment = VerticalAlignment.Center
                header.Children.Add(tb)
                Dim sort As New Polygon
                sort.Fill = Brushes.Black
                sort.FillRule = FillRule.EvenOdd
                sort.HorizontalAlignment = HorizontalAlignment.Right
                sort.VerticalAlignment = VerticalAlignment.Center
                sort.Points = New PointCollection(New List(Of Point) From {New Point(0, 4), New Point(8, 4), New Point(4, 0)})
                header.Children.Add(sort)
                bdr.Child = header

            ElseIf DirectCast(col.Tag, ListSortDirection) = ListSortDirection.Ascending Then
                Dim header As New Grid()

                Dim tb As New TextBlock()
                If bdr.Child IsNot Nothing Then
                    tb = TryCast(bdr.Child, TextBlock)
                End If
                tb.HorizontalAlignment = HorizontalAlignment.Left
                tb.VerticalAlignment = VerticalAlignment.Center
                bdr.Child = Nothing
                header.Children.Add(tb)
                Dim sort As New Polygon
                sort.HorizontalAlignment = HorizontalAlignment.Right
                sort.VerticalAlignment = VerticalAlignment.Center
                sort.Fill = Brushes.Black
                sort.FillRule = FillRule.EvenOdd
                sort.Points = New PointCollection(New List(Of Point) From {New Point(0, 0), New Point(8, 0), New Point(4, 4)})
                header.Children.Add(sort)
                bdr.Child = header
            End If
        End If

    End Sub
End Class






